/* -*- c++ -*- */
/* 
 * Copyright 2017 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */


#ifndef INCLUDED_DVBPARTS_L1_PRE_H
#define INCLUDED_DVBPARTS_L1_PRE_H

#include <DVBparts/api.h>
#include <gnuradio/block.h>

namespace gr {
  namespace DVBparts {

    /*!
     * \brief <+description of block+>
     * \ingroup DVBparts
     *
     */
    class DVBPARTS_API L1_Pre : virtual public gr::block
    {
     public:
      typedef boost::shared_ptr<L1_Pre> sptr;

      /*!
       * \brief Return a shared_ptr to a new instance of DVBparts::L1_Pre.
       *
       * To avoid accidental use of raw pointers, DVBparts::L1_Pre's
       * constructor is in a private implementation
       * class. DVBparts::L1_Pre::make is the public interface for
       * creating new instances.
       *
       * Output stream retagged with L1_Pre information, IFFT, time sync,
       * and time shift phase correction.
       *
       * \param gi initial guard interval 0:1/32, 1:1/16, ..., 7:undefined
       * \param gi_corr time shift for GI correction (ca. 2053 for 16k and 32k)
       */
      static sptr make( int gi=7, int gi_corr=2053 );

      virtual void set_gi( int gi ) = 0;
      virtual void set_gi_corr( int gi_corr ) = 0;

    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_L1_PRE_H */

