/* -*- c++ -*- */
/* 
 * Copyright 2017 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */


#ifndef INCLUDED_DVBPARTS_P1_DETECT_H
#define INCLUDED_DVBPARTS_P1_DETECT_H

#include <DVBparts/api.h>
#include <gnuradio/sync_block.h>

namespace gr {
  namespace DVBparts {

    /*!
     * \brief <+description of block+>
     * \ingroup DVBparts
     *
     */
    class DVBPARTS_API P1_Detect : virtual public gr::sync_block
    {
     public:
      typedef boost::shared_ptr<P1_Detect> sptr;

      /*!
       * \brief Return a shared_ptr to a new instance of DVBparts::P1_Detect.
       *
       * To avoid accidental use of raw pointers, DVBparts::P1_Detect's
       * constructor is in a private implementation
       * class. DVBparts::P1_Detect::make is the public interface for
       * creating new instances.
       *
       * \param TA size in samples of part A of P1 (1024 for DVB-T2)
       * \param TB size in samples of part B of P1 (482 for DVB-T2)
       * \param TC size in samples of part C of P1 (542 for DVB-T2)
       */
      static sptr make(int TA=1024, int TB=482, int TC=542);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_P1_DETECT_H */

