/* -*- c++ -*- */
/* 
 * Copyright 2017 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */


#ifndef INCLUDED_DVBPARTS_P1_VALIDATE_H
#define INCLUDED_DVBPARTS_P1_VALIDATE_H

#include <DVBparts/api.h>
#include <gnuradio/sync_block.h>

namespace gr {
  namespace DVBparts {

    /*!
     * \brief <+description of block+>
     * \ingroup DVBparts
     *
     */
    class DVBPARTS_API P1_Validate : virtual public gr::sync_block
    {
     public:
      typedef boost::shared_ptr<P1_Validate> sptr;

      /*!
       * \brief Return a shared_ptr to a new instance of DVBparts::P1_Validate.
       *
       * To avoid accidental use of raw pointers, DVBparts::P1_Validate's
       * constructor is in a private implementation
       * class. DVBparts::P1_Validate::make is the public interface for
       * creating new instances.
       *
       * Output stream tagged with P2 start tags.
       *
       * \param deltaT time shift to P1 start position (ca. -1368)
       * \param TH value for trigger threshold
       */
      static sptr make(int deltaT=-1368, float TH=0.1);

      virtual void set_deltaT( int deltaT ) = 0;
      virtual void set_threshold( float th ) = 0;

      virtual int get_deltaT() const = 0;
      virtual float get_threshold() const = 0;

    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_P1_VALIDATE_H */

