/* -*- c++ -*- */
/* 
 * Copyright 2018 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */


#ifndef INCLUDED_DVBPARTS_LDPC_DECODER_H
#define INCLUDED_DVBPARTS_LDPC_DECODER_H

#include <DVBparts/api.h>
#include <gnuradio/block.h>

namespace gr {
  namespace DVBparts {

    /*!
     * \brief <+description of block+>
     * \ingroup DVBparts
     *
     */
    class DVBPARTS_API ldpc_decoder : virtual public gr::block
    {
     public:
      typedef boost::shared_ptr<ldpc_decoder> sptr;

      /*!
       * \brief Return a shared_ptr to a new instance of DVBparts::ldpc_decoder.
       *
       * To avoid accidental use of raw pointers, DVBparts::ldpc_decoder's
       * constructor is in a private implementation
       * class. DVBparts::ldpc_decoder::make is the public interface for
       * creating new instances.
       *
       * LDPC decoded and stripped.
       *
       */
      static sptr make();
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_LDPC_DECODER_H */

