/* -*- c++ -*- */
/* 
 * Copyright 2018 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_DVBPARTS_L1_POST_IMPL_H
#define INCLUDED_DVBPARTS_L1_POST_IMPL_H

#include <DVBparts/L1-Post.h>

#include "L1Pre.h"
#include "bch.h"
#include "L1Post.h"
#include "PLPslice.h"

#define KBCH_1_2 7032
#define NBCH_1_2 7200

namespace gr {
  namespace DVBparts {

    class L1_Post_impl : public L1_Post
    {
     private:

      pmt::pmt_t d_msg_port;

      pmt::pmt_t L1TagValue;
      L1Pre l1pre;

      L1Post l1post;

      bool l1_randomize[KBCH_1_2];

      int bchcnt_in_blk[20];
      bch_control *BCHctl;

      int dvbt2_fftsize, fftsize, L1symbsize, L1pre_sz, L1post_sz;

      int demod_bits_cnt, demod_bits_cnt_in_blk;
      bool *demod_bits;

      int Kpost_ex_pad, Npost_fec_block, KL1_padding, Kpost, Ksig;
      unsigned char *L1_post_buffer; // Kpost_ex_pad

      int N_P2, C_P2, N_FC, C_FC, C_DATA;
      int C_PS, K_EXT, K_OFFSET, Kmin, Kmax;

      void init_l1_randomizer(void);
      void update_parameter(void);

      std::vector<PLPslice> PLPslices;
      void update_PLPslices(void);

     public:
      L1_Post_impl();
      ~L1_Post_impl();

      // Where all the action really happens
      void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      int general_work(int noutput_items,
           gr_vector_int &ninput_items,
           gr_vector_const_void_star &input_items,
           gr_vector_void_star &output_items);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_L1_POST_IMPL_H */

