/* -*- c++ -*- */
/* 
 * Copyright 2017 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_DVBPARTS_L1_PRE_IMPL_H
#define INCLUDED_DVBPARTS_L1_PRE_IMPL_H

#include <DVBparts/L1-Pre.h>

#include <gnuradio/fft/fft.h>
#include <gnuradio/dtv/dvb_config.h>
#include <gnuradio/dtv/dvbt2_config.h>

#include "L1Pre.h"
#include "bch.h"
#include "pilot_table.h"

#define MAX_CARRIERS 27841
#define MAX_FFT_SIZE 32768

enum dvbt2_carrier_type_t {
  DATA_CARRIER = 1,
  P2PILOT_CARRIER,
  P2PAPR_CARRIER,
  TRPAPR_CARRIER, 
  SCATTERED_CARRIER,
  CONTINUAL_CARRIER,
  P2PILOT_CARRIER_INVERTED,
  SCATTERED_CARRIER_INVERTED,
  CONTINUAL_CARRIER_INVERTED
};


namespace gr {
  namespace DVBparts {

    class L1_Pre_impl : public L1_Pre
    {
     private:
      int d_gi_ini, d_gi_corr, d_gi_size;

      pmt::pmt_t d_msg_port;

      int S1, S2;
      L1Pre l1pre;

      bool miso;
      int Ktotal, Kmin, Kmax, Kext, Koff, N_P2, C_P2;

      int fftsize;
      fft::fft_complex *d_fft_cfw, *d_fft_cbw;
      gr_complex c_norm[MAX_FFT_SIZE];

      float A_P2, p2_spec[MAX_FFT_SIZE];

      int prbs[MAX_CARRIERS];
      int pn_sequence[MAX_CHIPS];

      int p2_carrier_map[MAX_CARRIERS];

      int HevenP2[MAX_FFT_SIZE];
      int HoddP2[MAX_FFT_SIZE];

      bch_control *BCHctl;

      void init_prbs(void);
      void update_H(void);
      void update_cmap_P2(void);

      void update_gi_size( int gi );

     public:
      L1_Pre_impl( int gi, int gi_corr );
      ~L1_Pre_impl();

      void set_gi( int gi );
      void set_gi_corr( int gi_corr );

      // Where all the action really happens
      void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      int general_work(int noutput_items,
           gr_vector_int &ninput_items,
           gr_vector_const_void_star &input_items,
           gr_vector_void_star &output_items);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_L1_PRE_IMPL_H */

