/* -*- c++ -*- */
/* 
 * Copyright 2018 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#include <iostream>

#include <L1Post.h>

using namespace std;


#define bitcollect(C,S,bits,M) \
C = (S[bits++] ? M : 0 );\
for ( unsigned int mask=(M>>1); mask>0; mask>>=1 ) { if(S[bits++]) C |= mask; }\
\

void setup_L1post( L1Post &l1p, const L1Pre &l1pre, const unsigned char *L1P )
{
   int bits = 0;


   L1PostConfigurable &C = l1p.conf;

   bitcollect( C.sub_slices_per_frame, L1P, bits, 0x4000 );
   bitcollect( C.num_plp,              L1P, bits, 0x80 );
   bitcollect( C.num_aux,              L1P, bits, 0x8 );
   bitcollect( C.aux_config_rfu,       L1P, bits, 0x80 );

   for ( int i=0; i<l1pre.num_rf; ++i )
   {
      L1Prf &RF = C.RF[i];

      bitcollect( RF.rf_idx,    L1P, bits, 0x4 );
      bitcollect( RF.frequency, L1P, bits, 0x80000000 );
   }

   if ( l1pre.s2 & 0x1 )
   {
      bits += 4 + 22 + 8;
   }

   for ( int i=0; i<C.num_plp; ++i )
   {
      L1Pplp &P = C.PLP[i];

      bitcollect( P.plp_id, L1P, bits, 0x80 );
      bitcollect( P.plp_type, L1P, bits, 0x4 );
      bitcollect( P.plp_payload_type, L1P, bits, 0x10 );
      bitcollect( P.ff_flag, L1P, bits, 0x1 );
      bitcollect( P.first_rf_idx, L1P, bits, 0x4 );
      bitcollect( P.first_frame_index, L1P, bits, 0x80 );
      bitcollect( P.plp_group_id, L1P, bits, 0x80 );
      bitcollect( P.plp_cod, L1P, bits, 0x4 );
      bitcollect( P.plp_mod, L1P, bits, 0x4 );
      bitcollect( P.plp_rotation, L1P, bits, 0x1 );
      bitcollect( P.plp_fec_type, L1P, bits, 0x2 );
      bitcollect( P.plp_num_blocks_max, L1P, bits, 0x200 );
      bitcollect( P.frame_interval, L1P, bits, 0x80 );
      bitcollect( P.time_il_length, L1P, bits, 0x80 );
      bitcollect( P.time_il_type, L1P, bits, 0x1 );
      bitcollect( P.in_band_a_flag, L1P, bits, 0x1 );
      bitcollect( P.in_band_b_flag, L1P, bits, 0x1 );
      bits += 11; // reserved_1 ignored
      bitcollect( P.plp_mode, L1P, bits, 0x2 );
      bitcollect( P.static_flag, L1P, bits, 0x1 );
      bitcollect( P.static_padding_flag, L1P, bits, 0x1 );
   }

   bitcollect( C.fef_length_msb, L1P, bits, 0x2 );
   bits += 30; // reserved_2 ignored

   for ( int i=0; i<C.num_aux; ++i )
   {
      L1Paux &A = C.AUX[i];

      bitcollect( A.aux_stream_type,  L1P, bits, 0x8 );
      bitcollect( A.aux_private_conf, L1P, bits, 0x08000000 );
   }


   L1PostDynamic &D = l1p.dyn1;

   bitcollect( D.frame_idx, L1P, bits, 0x80 );
   bitcollect( D.sub_slice_interval, L1P, bits, 0x00200000 );
   bitcollect( D.type_2_start, L1P, bits, 0x00200000 );
   bitcollect( D.l1_change_counter, L1P, bits, 0x80 );
   bitcollect( D.start_rf_idx, L1P, bits, 0x4 );
   bits += 8; // reserved_1 ignored

   for ( int i=0; i<C.num_plp; ++i )
   {
      L1P_d_plp &P = D.PLP[i];

      bitcollect( P.plp_id, L1P, bits, 0x80 );
      bitcollect( P.plp_start, L1P, bits, 0x00200000 );
      bitcollect( P.plp_num_blocks, L1P, bits, 0x200 );
      bits += 8; // reserved_2 ignored
   }

   bits += 8; // reserved_3 ignored

   for ( int i=0; i<C.num_aux; ++i )
   {
      bits += 48; // aux_private_dyn ignored
   }


// dynamic repetition ignored, l1_repetition_flag == 1, size(dyn1)
// extension ignored


   bits = l1pre.l1_post_info_size;
   bitcollect( l1p.crc_32, L1P, bits, 0x80000000 );

}

void l1post_out( const L1Pre &l1pre, const L1Post &l1p )
{
   clog << "L1-post:" << endl;
   clog << "configurable" << endl;
   const L1PostConfigurable &C = l1p.conf;

   clog << " SUB_SLICES_PER_FRAME " << C.sub_slices_per_frame << endl;
   clog << " NUM_PLP " << C.num_plp << endl;
   clog << " NUM_AUX " << C.num_aux << endl;
   clog << " AUX_CONFIG_RFU " << C.aux_config_rfu << endl;
   for ( int i=0; i<l1pre.num_rf; ++i )
   {
      const L1Prf &RF = C.RF[i];
      clog << " RF_" << i << endl;

      clog << "  RF_IDX " << RF.rf_idx << endl;
      clog << "  FREQUENCY " << RF.frequency << endl;
   }

   for ( int i=0; i<C.num_plp; ++i )
   {
      const L1Pplp &P = C.PLP[i];
      clog << " PLP_" << i << endl;

      clog << "  PLP_ID " << P.plp_id << endl;
      clog << "  PLP_TYPE " << hex << "0x" << P.plp_type << endl;
      clog << "  PLP_PAYLOAD_TYPE " << hex << "0x" << P.plp_payload_type << dec << endl;
      clog << "  FF_FLAG " << P.ff_flag << endl;
      clog << "  FIRST_RF_IDX " << P.first_rf_idx << endl;
      clog << "  FIRST_FRAME_INDEX " << P.first_frame_index << endl;
      clog << "  PLP_GROUP_ID " << P.plp_group_id << endl;
      clog << "  PLP_COD " << P.plp_cod << endl;
      clog << "  PLP_MOD " << P.plp_mod << endl;
      clog << "  PLP_ROTATION " << P.plp_rotation << endl;
      clog << "  PLP_FEC_TYPE " << hex << "0x" << P.plp_fec_type << dec << endl;
      clog << "  PLP_NUM_BLOCKS_MAX " << P.plp_num_blocks_max << endl;
      clog << "  FRAME_INTERVAL " << P.frame_interval << endl;
      clog << "  TIME_IL_LENGTH " << P.time_il_length << endl;
      clog << "  TIME_IL_TYPE " << P.time_il_type << endl;
      clog << "  IN-BAND_A_FLAG " << P.in_band_a_flag << endl;
      clog << "  IN-BAND_B_FLAG " << P.in_band_b_flag << endl;
      // reserved_1 ignored
      clog << "  PLP_MODE " << P.plp_mode << endl;
      clog << "  STATIC_FLAG " << P.static_flag << endl;
      clog << "  STATIC_PADDING_FLAG " << P.static_padding_flag << endl;
   }

   clog << " FEF_LENGTH_MSB " << C.fef_length_msb << endl;
   // reserved_2 ignored

   for ( int i=0; i<C.num_aux; ++i )
   {
      const L1Paux &A = C.AUX[i];
      clog << " AUX_" << i << endl;

      clog << "  AUX_STREAM_TYPE " << A.aux_stream_type << endl;
      clog << "  AUX_PRIVATE_CONF " << A.aux_private_conf << endl;
   }

   clog << "dynamic" << endl;
   const L1PostDynamic &D = l1p.dyn1;

   clog << " FRAME_IDX " << D.frame_idx << endl;
   clog << " SUB_SLICE_INTERVAL " << D.sub_slice_interval << endl;
   clog << " TYPE_2_START " << D.type_2_start << endl;
   clog << " L1_CHANGE_COUNTER " << D.l1_change_counter << endl;
   clog << " START_RF_IDX " << D.start_rf_idx << endl;
   // reserved_1 ignored

   for ( int i=0; i<C.num_plp; ++i )
   {
      const L1P_d_plp &P = D.PLP[i];
      clog << " PLP_" << i << endl;

      clog << "  PLP_ID " << P.plp_id << endl;
      clog << "  PLP_START " << P.plp_start << endl;
      clog << "  PLP_NUM_BLOCKS " << P.plp_num_blocks << endl;
   }

   // reserved_3 ignored
   // aux_private_dyn ignored

// dynamic repetition ignored, l1_repetition_flag == 1, size(dyn1)
// extension ignored

   clog << "CRC32 " << l1p.crc_32 << endl;

}


void setup_L1Pplp_tag( pmt::pmt_t &L1PLPTagValue, const L1Pplp &plp )
{
   L1PLPTagValue = pmt::make_u32vector(20,0);
   pmt::u32vector_set(L1PLPTagValue,  0, plp.plp_id ); // 8
   pmt::u32vector_set(L1PLPTagValue,  1, plp.plp_type ); // 3
   pmt::u32vector_set(L1PLPTagValue,  2, plp.plp_payload_type); // 5
   pmt::u32vector_set(L1PLPTagValue,  3, plp.ff_flag); // 1
   pmt::u32vector_set(L1PLPTagValue,  4, plp.first_rf_idx); // 3
   pmt::u32vector_set(L1PLPTagValue,  5, plp.first_frame_index); // 8
   pmt::u32vector_set(L1PLPTagValue,  6, plp.plp_group_id); // 8
   pmt::u32vector_set(L1PLPTagValue,  7, plp.plp_cod); // 3
   pmt::u32vector_set(L1PLPTagValue,  8, plp.plp_mod); // 3
   pmt::u32vector_set(L1PLPTagValue,  9, plp.plp_rotation); // 1
   pmt::u32vector_set(L1PLPTagValue, 10, plp.plp_fec_type); // 2
   pmt::u32vector_set(L1PLPTagValue, 11, plp.plp_num_blocks_max); // 10
   pmt::u32vector_set(L1PLPTagValue, 12, plp.frame_interval); // 8
   pmt::u32vector_set(L1PLPTagValue, 13, plp.time_il_length); // 8
   pmt::u32vector_set(L1PLPTagValue, 14, plp.time_il_type); // 1
   pmt::u32vector_set(L1PLPTagValue, 15, plp.in_band_a_flag); // 1
   pmt::u32vector_set(L1PLPTagValue, 16, plp.in_band_b_flag); // 1
// reserved_1 ignored 11
   pmt::u32vector_set(L1PLPTagValue, 17, plp.plp_mode); // 2
   pmt::u32vector_set(L1PLPTagValue, 18, plp.static_flag); // 1
   pmt::u32vector_set(L1PLPTagValue, 19, plp.static_padding_flag); // 1
}

void setup_L1Pplp( L1Pplp &plp, const pmt::pmt_t &L1PLPTagValue )
{
   plp.plp_id = pmt::u32vector_ref(L1PLPTagValue,0); // 8
   plp.plp_type = pmt::u32vector_ref(L1PLPTagValue,1); // 3
   plp.plp_payload_type = pmt::u32vector_ref(L1PLPTagValue,2); // 5
   plp.ff_flag = pmt::u32vector_ref(L1PLPTagValue,3); // 1
   plp.first_rf_idx = pmt::u32vector_ref(L1PLPTagValue,4); // 3
   plp.first_frame_index = pmt::u32vector_ref(L1PLPTagValue,5); // 8
   plp.plp_group_id = pmt::u32vector_ref(L1PLPTagValue,6); // 8
   plp.plp_cod = pmt::u32vector_ref(L1PLPTagValue,7); // 3
   plp.plp_mod = pmt::u32vector_ref(L1PLPTagValue,8); // 3
   plp.plp_rotation = pmt::u32vector_ref(L1PLPTagValue,9); // 1
   plp.plp_fec_type = pmt::u32vector_ref(L1PLPTagValue,10); // 2
   plp.plp_num_blocks_max = pmt::u32vector_ref(L1PLPTagValue,11); // 10
   plp.frame_interval = pmt::u32vector_ref(L1PLPTagValue,12); // 8
   plp.time_il_length = pmt::u32vector_ref(L1PLPTagValue,13); // 8
   plp.time_il_type = pmt::u32vector_ref(L1PLPTagValue,14); // 1
   plp.in_band_a_flag = pmt::u32vector_ref(L1PLPTagValue,15); // 1
   plp.in_band_b_flag = pmt::u32vector_ref(L1PLPTagValue,16); // 1
// reserved_1 ignored 11
   plp.plp_mode = pmt::u32vector_ref(L1PLPTagValue,17); // 2
   plp.static_flag = pmt::u32vector_ref(L1PLPTagValue,18); // 1
   plp.static_padding_flag = pmt::u32vector_ref(L1PLPTagValue,19); // 1
}

