/* -*- c++ -*- */
/* 
 * Copyright 2018 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef L1POST_H
#define L1POST_H

#include <pmt/pmt.h>

#include "L1Pre.h"


typedef struct {
   unsigned int rf_idx; // 3
   unsigned int frequency; // 32
} L1Prf;

typedef struct {
   unsigned int plp_id; // 8
   unsigned int plp_type; // 3
   unsigned int plp_payload_type; // 5
   unsigned int ff_flag; // 1
   unsigned int first_rf_idx; // 3
   unsigned int first_frame_index; // 8
   unsigned int plp_group_id; // 8
   unsigned int plp_cod; // 3
   unsigned int plp_mod; // 3
   unsigned int plp_rotation; // 1
   unsigned int plp_fec_type; // 2
   unsigned int plp_num_blocks_max; // 10
   unsigned int frame_interval; // 8
   unsigned int time_il_length; // 8
   unsigned int time_il_type; // 1
   unsigned int in_band_a_flag; // 1
   unsigned int in_band_b_flag; // 1
// reserved_1 ignored 11
   unsigned int plp_mode; // 2
   unsigned int static_flag; // 1
   unsigned int static_padding_flag; // 1
} L1Pplp;

typedef struct {
   unsigned int aux_stream_type; // 4
   unsigned int aux_private_conf; // 28
} L1Paux;

typedef struct {
   unsigned int sub_slices_per_frame; // 15
   unsigned int num_plp; // 8
   unsigned int num_aux; // 4
   unsigned int aux_config_rfu; // 8

   L1Prf RF[7]; // num_rf:3

// FEF ignored, S2 == xxx1, 4 + 22 + 8

   L1Pplp PLP[256]; // num_plp:8

   unsigned int fef_length_msb; // 2
// reserved_2 ignored 30
   
   L1Paux AUX[16]; // num_aux:4

} L1PostConfigurable;


typedef struct {
   unsigned int plp_id; // 8
   unsigned int plp_start; // 22
   unsigned int plp_num_blocks; // 10
// reserved_2 ignored 8
} L1P_d_plp;

typedef struct {
// aux_private_dyn ignored 48
} L1P_d_aux;

typedef struct {
   unsigned int frame_idx; // 8
   unsigned int sub_slice_interval; // 22
   unsigned int type_2_start; // 22
   unsigned int l1_change_counter; // 8
   unsigned int start_rf_idx; // 3
// reserved_1 ignored 8

   L1P_d_plp PLP[256]; // num_plp:8

// reserved_3 ignored 8

   L1P_d_aux AUX[16]; // num_aux:4

} L1PostDynamic;


typedef struct {
   L1PostConfigurable conf;
   L1PostDynamic dyn1;
// dynamic repetition ignored, l1_repetition_flag == 1, size(dyn1)
// extension ignored
   unsigned int crc_32; // 32
} L1Post;


void setup_L1post( L1Post &l1p, const L1Pre &l1pre, const unsigned char *L1P );

void l1post_out( const L1Pre &l1pre, const L1Post &l1p );


void setup_L1Pplp_tag( pmt::pmt_t &L1PLPValue, const L1Pplp &plp );

void setup_L1Pplp( L1Pplp &plp, const pmt::pmt_t &L1PLPValue );


#endif
