/* -*- c++ -*- */
/* 
 * Copyright 2017 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#include "L1Pre.h"

#include <iostream>

using namespace std;


void l1pre_out( const L1Pre &l1p )
{
   clog << "L1-pre:" << endl;
   clog << " TYPE " << hex << "0x" << l1p.type << dec << endl;
   clog << " BWT_EXT " << l1p.bwt_ext << endl;
   clog << " S1 " << l1p.s1 << endl;
   clog << " S2 " << l1p.s2 << endl;
   clog << " L1_REPETITION_FLAG " << l1p.l1_repetition_flag << endl;
   clog << " GUARD_INTERVALL " << l1p.guard_interval << endl;
   clog << " PAPR " << l1p.papr << endl;
   clog << " L1_MOD " << l1p.l1_mod << endl;
   clog << " L1_COD " << l1p.l1_cod << endl;
   clog << " L1_FEC_TYPE " << l1p.l1_fec_type << endl;
   clog << " L1_POST_SIZE " << l1p.l1_post_size << endl;
   clog << " L1_POST_INFO_SIZE " << l1p.l1_post_info_size << endl;
   clog << " PILOT_PATTERN " << l1p.pilot_pattern << endl;
   clog << " TX_ID_AVAILABILITY " << l1p.tx_id_availability << endl;

   clog << " CELL_ID " << hex << "0x" << l1p.cell_id << endl;
   clog << " NETWORK_ID " << hex << "0x" << l1p.network_id << endl;
   clog << " T2_SYSTEM_ID " << hex << "0x" << l1p.t2_system_id << dec << endl;

   clog << " NUM_T2_FRAMES " << l1p.num_t2_frames << endl;
   clog << " NUM_DATA_SYMBOLS " << l1p.num_data_symbols << endl;
   clog << " REGEN_FLAG " << l1p.regen_flag << endl;
   clog << " L1_POST_EXTENSION " << l1p.l1_post_extension << endl;
   clog << " NUM_RF " << l1p.num_rf << endl;
   clog << " CURRENT_RF_INDEX " << l1p.current_rf_index << endl;
   clog << " T2_VERSION " << l1p.t2_version << endl;
   clog << " L1_POST_SCRAMBLED " << l1p.l1_post_scrambled << endl;
   clog << " T2_BASE_LITE " << l1p.t2_base_lite << endl;
   clog << " RESERVED " << hex << "0x" << l1p.reserved << dec << endl;
   clog << " CRC_32 "  << hex << "0x" << l1p.crc_32 << dec << endl;
}


void setup_L1pre( L1Pre &l1p, const unsigned char *L1pre )
{
   int bits = 0;

   l1p.type = (L1pre[bits++] ? 0x80 : 0 );
   for ( unsigned int mask=0x40; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.type |= mask; }

   l1p.bwt_ext = (L1pre[bits++] ? 0x1 : 0 );

   l1p.s1 = (L1pre[bits++] ? 0x4 : 0 );
   for ( unsigned int mask=0x2; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.s1 |= mask; }

   l1p.s2 = (L1pre[bits++] ? 0x8 : 0 );
   for ( unsigned int mask=0x4; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.s2 |= mask; }

   l1p.l1_repetition_flag = (L1pre[bits++] ? 0x1 : 0 );

   l1p.guard_interval = (L1pre[bits++] ? 0x4 : 0 );
   for ( unsigned int mask=0x2; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.guard_interval |= mask; }

   l1p.papr = (L1pre[bits++] ? 0x8 : 0 );
   for ( unsigned int mask=0x4; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.papr |= mask; }

   l1p.l1_mod = (L1pre[bits++] ? 0x8 : 0 );
   for ( unsigned int mask=0x4; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.l1_mod |= mask; }

   l1p.l1_cod  = (L1pre[bits++] ? 0x2 : 0 );
   if (L1pre[bits++]) { l1p.l1_cod |= 0x1; }

   l1p.l1_fec_type  = (L1pre[bits++] ? 0x2 : 0 );
   if (L1pre[bits++]) { l1p.l1_fec_type |= 0x1; }

   l1p.l1_post_size = (L1pre[bits++] ? 0x20000 : 0 );
   for ( unsigned int mask=0x10000; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.l1_post_size |= mask; }

   l1p.l1_post_info_size = (L1pre[bits++] ? 0x20000 : 0 );
   for ( unsigned int mask=0x10000; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.l1_post_info_size |= mask; }

   l1p.pilot_pattern = (L1pre[bits++] ? 0x8 : 0 );
   for ( unsigned int mask=0x4; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.pilot_pattern |= mask; }

   l1p.tx_id_availability = (L1pre[bits++] ? 0x80 : 0 );
   for ( unsigned int mask=0x40; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.tx_id_availability |= mask; }

   l1p.cell_id = (L1pre[bits++] ? 0x8000 : 0 );
   for ( unsigned int mask=0x4000; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.cell_id |= mask; }

   l1p.network_id = (L1pre[bits++] ? 0x8000 : 0 );
   for ( unsigned int mask=0x4000; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.network_id |= mask; }

   l1p.t2_system_id = (L1pre[bits++] ? 0x8000 : 0 );
   for ( unsigned int mask=0x4000; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.t2_system_id |= mask; }

   l1p.num_t2_frames = (L1pre[bits++] ? 0x80 : 0 );
   for ( unsigned int mask=0x40; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.num_t2_frames |= mask; }

   l1p.num_data_symbols = (L1pre[bits++] ? 0x800 : 0 );
   for ( unsigned int mask=0x400; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.num_data_symbols |= mask; }

   l1p.regen_flag = (L1pre[bits++] ? 0x4 : 0 );
   for ( unsigned int mask=0x2; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.regen_flag |= mask; }

   l1p.l1_post_extension = (L1pre[bits++] ? 0x1 : 0 );

   l1p.num_rf = (L1pre[bits++] ? 0x4 : 0 );
   for ( unsigned int mask=0x2; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.num_rf |= mask; }

   l1p.current_rf_index = (L1pre[bits++] ? 0x4 : 0 );
   for ( unsigned int mask=0x2; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.current_rf_index |= mask; }

   l1p.t2_version = (L1pre[bits++] ? 0x8 : 0 );
   for ( unsigned int mask=0x4; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.t2_version |= mask; }

   l1p.l1_post_scrambled = (L1pre[bits++] ? 0x1 : 0 );

   l1p.t2_base_lite = (L1pre[bits++] ? 0x1 : 0 );

   l1p.reserved = (L1pre[bits++] ? 0x8 : 0 );
   for ( unsigned int mask=0x4; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.reserved |= mask; }

   l1p.crc_32 = (L1pre[bits++] ? 0x80000000 : 0 );
   for ( unsigned int mask=0x40000000; mask>0; mask>>=1 ) { if(L1pre[bits++]) l1p.crc_32 |= mask; }


if ( l1p.t2_version > 0 )
{ cerr << "L1-ACE not implemented!" << endl; }


}


void setup_L1tag( pmt::pmt_t &L1TagValue, const L1Pre &l1p )
{
   L1TagValue = pmt::make_u32vector(28,0);
   pmt::u32vector_set(L1TagValue, 0, l1p.type ); // 8
   pmt::u32vector_set(L1TagValue, 1, l1p.bwt_ext ); // 1
   pmt::u32vector_set(L1TagValue, 2, l1p.s1 ); // 3
   pmt::u32vector_set(L1TagValue, 3, l1p.s2 ); // 4
   pmt::u32vector_set(L1TagValue, 4, l1p.l1_repetition_flag ); // 1
   pmt::u32vector_set(L1TagValue, 5, l1p.guard_interval ); // 3
   pmt::u32vector_set(L1TagValue, 6, l1p.papr ); // 4
   pmt::u32vector_set(L1TagValue, 7, l1p.l1_mod ); // 4
   pmt::u32vector_set(L1TagValue, 8, l1p.l1_cod ); // 2
   pmt::u32vector_set(L1TagValue, 9, l1p.l1_fec_type ); // 2 
   pmt::u32vector_set(L1TagValue,10, l1p.l1_post_size ); // 18
   pmt::u32vector_set(L1TagValue,11, l1p.l1_post_info_size ); // 18
   pmt::u32vector_set(L1TagValue,12, l1p.pilot_pattern ); // 4
   pmt::u32vector_set(L1TagValue,13, l1p.tx_id_availability ); // 8
   pmt::u32vector_set(L1TagValue,14, l1p.cell_id ); // 16
   pmt::u32vector_set(L1TagValue,15, l1p.network_id ); // 16
   pmt::u32vector_set(L1TagValue,16, l1p.t2_system_id ); // 16
   pmt::u32vector_set(L1TagValue,17, l1p.num_t2_frames ); // 8
   pmt::u32vector_set(L1TagValue,18, l1p.num_data_symbols ); // 12
   pmt::u32vector_set(L1TagValue,19, l1p.regen_flag ); // 3
   pmt::u32vector_set(L1TagValue,20, l1p.l1_post_extension ); // 1
   pmt::u32vector_set(L1TagValue,21, l1p.num_rf ); // 3
   pmt::u32vector_set(L1TagValue,22, l1p.current_rf_index ); // 3
   pmt::u32vector_set(L1TagValue,23, l1p.t2_version ); // 4
   pmt::u32vector_set(L1TagValue,24, l1p.l1_post_scrambled ); // 1
   pmt::u32vector_set(L1TagValue,25, l1p.t2_base_lite ); // 1
   pmt::u32vector_set(L1TagValue,26, l1p.reserved ); // 4
   pmt::u32vector_set(L1TagValue,27, l1p.crc_32 ); // 32
}

void setup_L1pre( L1Pre &l1p, const pmt::pmt_t &L1TagValue )
{
   l1p.type = pmt::u32vector_ref(L1TagValue,0);  // 8
   l1p.bwt_ext = pmt::u32vector_ref(L1TagValue,1); // 1
   l1p.s1 = pmt::u32vector_ref(L1TagValue,2); // 3
   l1p.s2 = pmt::u32vector_ref(L1TagValue,3); // 4            
   l1p.l1_repetition_flag = pmt::u32vector_ref(L1TagValue,4); // 1
   l1p.guard_interval = pmt::u32vector_ref(L1TagValue,5); // 3
   l1p.papr = pmt::u32vector_ref(L1TagValue,6); // 4
   l1p.l1_mod = pmt::u32vector_ref(L1TagValue,7); // 4          
   l1p.l1_cod = pmt::u32vector_ref(L1TagValue,8); // 2            
   l1p.l1_fec_type = pmt::u32vector_ref(L1TagValue,9); // 2       
   l1p.l1_post_size = pmt::u32vector_ref(L1TagValue,10); // 18           
   l1p.l1_post_info_size = pmt::u32vector_ref(L1TagValue,11); // 18       
   l1p.pilot_pattern = pmt::u32vector_ref(L1TagValue,12); // 4
   l1p.tx_id_availability = pmt::u32vector_ref(L1TagValue,13); // 8       
   l1p.cell_id = pmt::u32vector_ref(L1TagValue,14); // 16
   l1p.network_id = pmt::u32vector_ref(L1TagValue,15); // 16         
   l1p.t2_system_id = pmt::u32vector_ref(L1TagValue,16); // 16          
   l1p.num_t2_frames = pmt::u32vector_ref(L1TagValue,17); // 8            
   l1p.num_data_symbols = pmt::u32vector_ref(L1TagValue,18); // 12        
   l1p.regen_flag = pmt::u32vector_ref(L1TagValue,19); // 3
   l1p.l1_post_extension = pmt::u32vector_ref(L1TagValue,20); // 1     
   l1p.num_rf = pmt::u32vector_ref(L1TagValue,21); // 3
   l1p.current_rf_index = pmt::u32vector_ref(L1TagValue,22); // 3  
   l1p.t2_version = pmt::u32vector_ref(L1TagValue,23); // 4
   l1p.l1_post_scrambled = pmt::u32vector_ref(L1TagValue,24); // 1     
   l1p.t2_base_lite = pmt::u32vector_ref(L1TagValue,25); // 1
   l1p.reserved = pmt::u32vector_ref(L1TagValue,26); // 4
   l1p.crc_32 = pmt::u32vector_ref(L1TagValue,27); // 32
}

