/* -*- c++ -*- */
/* 
 * Copyright 2017 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_DVBPARTS_P1_VALIDATE_IMPL_H
#define INCLUDED_DVBPARTS_P1_VALIDATE_IMPL_H

#include <DVBparts/P1_Validate.h>

#include <gnuradio/fft/fft.h>


namespace gr {
  namespace DVBparts {

    class P1_Validate_impl : public P1_Validate
    {
     private:
      unsigned int d_TA, d_TB, d_TC;
      int d_deltaT;
      float d_TH;

      pmt::pmt_t d_msg_port;

      bool S1_pattern[8][64];
      bool S2_pattern[16][256];

      gr_complex *p1spec;

      fft::fft_complex *d_fft_cfw;
      fft::fft_real_fwd *d_fft_ffw;
      fft::fft_real_rev *d_fft_fbw;

      signed char p1_randomize[384];
      void init_p1_randomizer(void);

     public:
      P1_Validate_impl(int deltaT, float TH);
      ~P1_Validate_impl();

      void set_deltaT( int deltaT );
      void set_threshold( float th );

      int get_deltaT() const { return(d_deltaT); }
      float get_threshold() const { return(d_TH); }

      // Where all the action really happens
      int work(int noutput_items,
         gr_vector_const_void_star &input_items,
         gr_vector_void_star &output_items);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_P1_VALIDATE_IMPL_H */

