/* -*- c++ -*- */
/* 
 * Copyright 2018 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_DVBPARTS_CELL_DEINTERLEAVER_IMPL_H
#define INCLUDED_DVBPARTS_CELL_DEINTERLEAVER_IMPL_H

#include <DVBparts/cell_deinterleaver.h>
#include <gnuradio/dtv/dvb_config.h>

#include "L1Post.h"


#define FRAME_SIZE_NORMAL 64800


namespace gr {
  namespace DVBparts {

    class cell_deinterleaver_impl : public cell_deinterleaver
    {
     private:
      int cell_size;
      int pn_degree;
      int ti_blocks;
      int fec_blocks;
      int permutations[32768];

      pmt::pmt_t PLPTagValue;
      L1Pplp l1plp;

      unsigned int ci_buffer_offset;
      gr_complex ci_buffer[ FRAME_SIZE_NORMAL ];  // 64800

     public:
      cell_deinterleaver_impl();
      ~cell_deinterleaver_impl();

      void setup_permutations();

      // Where all the action really happens
      void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      int general_work(int noutput_items,
           gr_vector_int &ninput_items,
           gr_vector_const_void_star &input_items,
           gr_vector_void_star &output_items);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_CELL_DEINTERLEAVER_IMPL_H */

