/* -*- c++ -*- */
/* 
 * Copyright 2021 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_DVBPARTS_LDPC_DECODER_IMPL_H
#define INCLUDED_DVBPARTS_LDPC_DECODER_IMPL_H

#include <gnuradio/dtv/dvb_ldpc_bb.h>
#include <DVBparts/ldpc_decoder.h>

#include "L1Post.h"


#include "ldpc_dvb_tables.h"


#define FRAME_SIZE_NORMAL 64800
#define FRAME_SIZE_SHORT  16200

#define UFRAME_MAX_SIZE_NORMAL 54000
#define UFRAME_MAX_SIZE_SHORT  13320

#define MAX_SYN_SIZE 32400

#define MAX_CHUNK_SIZE 4
#define MAX_ITER_COUNT 10


namespace gr {
  namespace DVBparts {

    class SparseHMatrix
    {
       public:
          int N, K, Q;
          unsigned int *row;
          unsigned int *value;

          std::set<int> *bits_in_syndrome;

          template <typename TABLE>
          void update_sparse_matrix();

          SparseHMatrix()
          : N(0), K(0), Q(0), row(NULL), value(NULL),
            bits_in_syndrome(NULL)
          {
             update_sparse_matrix<DVB_T2X_TABLE_NULL>();
          }

          ~SparseHMatrix()
          {
             delete [] row; delete [] value;
             delete [] bits_in_syndrome;
          }

    };

    class ldpc_decoder_impl : public ldpc_decoder
    {
     private:
      float forecast_f;

      pmt::pmt_t PLPTagValue;
      L1Pplp l1plp;

      unsigned int fec_buffer_offset;
      bool fec_buffer_n[ FRAME_SIZE_NORMAL * MAX_CHUNK_SIZE ];
      int fec_decoding( int thread );

      bool syndrome_d_n[ MAX_SYN_SIZE * MAX_CHUNK_SIZE ],
           syndrome_p_n[ MAX_SYN_SIZE * MAX_CHUNK_SIZE ];
      uint8_t error_counter_n[ FRAME_SIZE_NORMAL * MAX_CHUNK_SIZE ];

      SparseHMatrix H;

      void update_tables();

      int decode( int max_iter, int thread );

     public:
      ldpc_decoder_impl();
      ~ldpc_decoder_impl();

      // Where all the action really happens
      void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      int general_work(int noutput_items,
           gr_vector_int &ninput_items,
           gr_vector_const_void_star &input_items,
           gr_vector_void_star &output_items);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_LDPC_DECODER_IMPL_H */

