/* -*- c++ -*- */
/* 
 * Copyright 2018 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_DVBPARTS_LDPC_DECODER_IMPL_H
#define INCLUDED_DVBPARTS_LDPC_DECODER_IMPL_H

#include <gnuradio/dtv/dvb_ldpc_bb.h>
#include <DVBparts/ldpc_decoder_soft.h>

#include "L1Post.h"

#define FRAME_SIZE_NORMAL 64800
#define FRAME_SIZE_MEDIUM 32400
#define FRAME_SIZE_SHORT  16200


#include "ldpc/ldpc.hh"
#include "ldpc/dvb_s2_tables.hh"
#include "ldpc/dvb_t2_tables.hh"
#include "ldpc/simd.hh"
#include "ldpc/algorithms.hh"

#include "dvb_t2x_tables.hh"


#define FACTOR 2

#ifdef __AVX2__
const int SIZEOF_SIMD = 32;
#else
const int SIZEOF_SIMD = 16;
#endif

typedef int8_t code_type;
const int SIMD_WIDTH = SIZEOF_SIMD / sizeof(code_type);
typedef SIMD<code_type, SIMD_WIDTH> simd_type;

#if 0
#include "ldpc/flooding_decoder.hh"
typedef SelfCorrectedUpdate<simd_type> update_type;
typedef MinSumAlgorithm<simd_type, update_type> algorithm_type;
const int TRIALS = 50;
#else
#include "ldpc/layered_decoder.hh"
typedef NormalUpdate<simd_type> update_type;
typedef OffsetMinSumAlgorithm<simd_type, update_type, FACTOR> algorithm_type;
const int TRIALS = 25;
#endif


namespace gr {
  namespace DVBparts {

    class ldpc_decoder_soft_impl : public ldpc_decoder_soft
    {
     private:
      float forecast_f;
      int NBCH, NLDPC, plen;

      pmt::pmt_t PLPTagValue;
      L1Pplp l1plp;

      unsigned int q_val;

      unsigned int fec_buffer_offset;
      int8_t fec_buffer[ FRAME_SIZE_NORMAL * SIZEOF_SIMD ];  // FRAME_SIZE_NORMAL * SIZEOF_SIMD


      unsigned int total_trials;
      LDPCInterface *ldpc;
      LDPCDecoder<simd_type, algorithm_type> decode;
      void *aligned_buffer;


      void update_tables();
      
     public:
      ldpc_decoder_soft_impl();
      ~ldpc_decoder_soft_impl();

      // Where all the action really happens
      void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      int general_work(int noutput_items,
           gr_vector_int &ninput_items,
           gr_vector_const_void_star &input_items,
           gr_vector_void_star &output_items);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_LDPC_DECODER_IMPL_H */

