/* -*- c++ -*- */
/* 
 * Copyright 2017 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_DVBPARTS_PILOT_REMOVE_RENORM_IMPL_H
#define INCLUDED_DVBPARTS_PILOT_REMOVE_RENORM_IMPL_H

#include <DVBparts/pilot_remove_renorm.h>

#include <gnuradio/dtv/dvb_config.h>
#include <gnuradio/dtv/dvbt2_config.h>

#include "L1Pre.h"
#include "pilot_table.h"


#define MAX_CARRIERS 27841
#define MAX_FFT_SIZE 32768

enum dvbt2_carrier_type_t {
  DATA_CARRIER = 1,
  P2PILOT_CARRIER,
  P2PAPR_CARRIER,
  TRPAPR_CARRIER, 
  SCATTERED_CARRIER,
  CONTINUAL_CARRIER,
  P2PILOT_CARRIER_INVERTED,
  SCATTERED_CARRIER_INVERTED,
  CONTINUAL_CARRIER_INVERTED
};


namespace gr {
  namespace DVBparts {

    class pilot_remove_renorm_impl : public pilot_remove_renorm
    {
     private:
      pmt::pmt_t L1TagValue;
      L1Pre l1pre;

      int dvbt2_fftsize, fftsize;

      gr_complex c_norm[MAX_FFT_SIZE];

      gr_complex p2_bpsk[2];
      gr_complex sp_bpsk[2];
      gr_complex cp_bpsk[2];
      gr_complex p2_bpsk_inverted[2];
      gr_complex sp_bpsk_inverted[2];
      gr_complex cp_bpsk_inverted[2];

      int prbs[MAX_CARRIERS];
      int pn_sequence[MAX_CHIPS];

      unsigned char carrier_map[MAX_CHIPS][MAX_CARRIERS];
      int k2idx[MAX_CARRIERS];

      int HevenP2[MAX_FFT_SIZE];
      int HoddP2[MAX_FFT_SIZE];
      int Heven[MAX_FFT_SIZE];
      int Hodd[MAX_FFT_SIZE];
      int HevenFC[MAX_FFT_SIZE];
      int HoddFC[MAX_FFT_SIZE];

      int N_P2, C_P2, N_FC, C_FC, C_DATA;
      int C_PS, K_EXT, K_OFFSET, Kmin, Kmax;
      int dx, dy;
      int miso, miso_group;

      float forecast_f;

      void init_prbs(void);
      void init_pilots(unsigned char *data_carrier_map, int symbol);

      void update_cmap(void);
      void update_H(void);

     public:
      pilot_remove_renorm_impl();
      ~pilot_remove_renorm_impl();

      // Where all the action really happens
      void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      int general_work(int noutput_items,
           gr_vector_int &ninput_items,
           gr_vector_const_void_star &input_items,
           gr_vector_void_star &output_items);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_PILOT_REMOVE_RENORM_IMPL_H */

