/* -*- c++ -*- */
/* 
 * Copyright 2018 Ingo D. Rullhusen <d01c@freeshell.de>.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_DVBPARTS_BB2PAYLOAD_IMPL_H
#define INCLUDED_DVBPARTS_BB2PAYLOAD_IMPL_H

#include <DVBparts/BB2payload.h>

#include "L1Post.h"


typedef struct
{
   unsigned char ts_gs;   // 2
   unsigned char sis_mis; // 1
   unsigned char ccm_acm; // 1
   unsigned char issyi;   // 1
   unsigned char npd;     // 1
   unsigned char ext;     // 2
} MATYPE1;

typedef struct
{
   MATYPE1 matype_1;
   unsigned char matype_2;
} MATYPE;

typedef struct
{
   MATYPE matype;
   unsigned int upl;   // *  ISSY 2MSB
   unsigned int dfl;

   unsigned char sync; // *  ISSY 1LSB

   unsigned int syncd;
   unsigned char crc8mode;

   unsigned char MODE; // 0  1  (3...255 NOK)
   unsigned char SYNC[8];
   unsigned int O_UPL_nsb, UPL;

} BBHEADER;            // NM HEM


namespace gr {
  namespace DVBparts {

    class BB2payload_impl : public BB2payload
    {
     private:
      float forecast_f;

      pmt::pmt_t d_msg_port;

      BBHEADER bbheader;

      bool setup_header( const unsigned char *in );
      void bbheader_out() const;

     public:
      BB2payload_impl();
      ~BB2payload_impl();

      // Where all the action really happens
///      void forecast (int noutput_items, gr_vector_int &ninput_items_required);

      int general_work(int noutput_items,
           gr_vector_int &ninput_items,
           gr_vector_const_void_star &input_items,
           gr_vector_void_star &output_items);
    };

  } // namespace DVBparts
} // namespace gr

#endif /* INCLUDED_DVBPARTS_BB2PAYLOAD_IMPL_H */

